<?php

namespace Drupal\tigo_portability\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\tigo_portability\Services\Token;
use Drupal\tigo_portability\Model\TokenModel;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\tigo_portability\Services\Pin;

/**
 * Clase que que genera un API para solicitud de NIP.
 *
 * @package Drupal\tigo_portability
 * @license https://transacciones.tigo.com.co/servicios/portabilidad  License 3.01
 * @link https://transacciones.tigo.com.co/servicios/portabilidad
 */
class ApiPortabilityController extends ControllerBase {

  private $tokenModel;
  private $token;
  private $nip;

  /**
   * Constructor de la clase.
   */
  public function __construct(
    Token $token_service,
    TokenModel $token_model,
    Pin $pin_service
  ) {
    $this->token = $token_service;
    $this->tokenModel = $token_model;
    $this->nip = $pin_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tigo_portability.token'),
      $container->get('tigo_portability.token_model'),
      $container->get('tigo_portability.pin')
    );
  }

  /**
   * Solicitad el donorId de un msisdn.
   *
   * @param string $token
   *   Token de verificación para consumir el servicio.
   * @param string $msisdn
   *   Línea para consultar el donorId.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   Retorna la respuesta en un formato Json
   */
  public function getDonor($token, $msisdn) {
    if (!$this->token->validateToken($token)) {
      $code = 401;
      $message = 'Token no valido';
    }
    else {
      $service = \Drupal::service('tigo_portability.operator_donor');
      $service->operator(['msisdn' => $msisdn]);
      $responseWS = $service->getResponse();
      $code = 200;
      $message = [
        'payload' => [
          'donorId' => $responseWS->payload->donorId,
          'name' => $responseWS->payload->name,
          'operatorABD' => $responseWS->payload->operatorABD,
        ],
      ];
    }
    return JsonResponse::create([
      'message' => $message,
    ], $code)->setSharedMaxAge(300);
  }

  /**
   * Solicitad el NIP de un msisdn.
   *
   * @param string $token
   *   Token de verificación para consumir el servicio.
   * @param string $donorid
   *   Operador al que pertenece la línea.
   * @param string $msisdn
   *   Línea para solicitar el NIP.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   Retorna la respuesta en un formato Json
   */
  public function getNip($token, $donorid, $msisdn) {
    $access = $this->haveAccess();
    if ($access['code'] == 200) {
      if (!$this->token->validateToken($token)) {
        $code = '401';
        $message = 'El token no es valido';
      }
      else {
        $data = [];
        $data['msisdn'] = $msisdn;
        $data['donorid'] = $donorid;
        $this->nip->pin($data);
        $responseWS = $this->nip->getResponse();
        if ($responseWS === NULL) {
          $code = '200';
          $message = 'Solicitud exitosa';
        }
        elseif (isset($responseWS)) {
          $resposeDetail = explode(':', $responseWS);
          $code = $resposeDetail[0];
          $message = $resposeDetail[1];
        }
      }
    }
    else {
      $code = $access['code'];
      $message = $access['message'];
    }
    $nip_response = $code . ' - ' . $message;
    $data_insert = [
      'created' => date('Y-m-d H:i:s'),
      'msisdn' => $msisdn,
      'domain' => isset($_SERVER['HTTP_REFERER'])
      ? $_SERVER['HTTP_REFERER']
      : '',
      'nip_response' => isset($responseWS)
      ? $responseWS
      : $nip_response,
      'acceso' => 'PERMITIDO',
      'url_requested' => $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'],
    ];
    $this->tokenModel->setNipRequest($data_insert);
    return JsonResponse::create([
      'message' => $message,
    ], $code)->setSharedMaxAge(300);
  }

  /**
   * Verifica si la solicitud es de otro dominio.
   *
   * @return array
   *   Con la respuesta de validación del dominio.
   */
  public function haveAccess() {
    $return = [
      'code' => 200,
      'message' => 'Accesos concedidos',
    ];
    if (!isset($_SERVER['HTTP_REFERER']) ||
      $_SERVER['HTTP_REFERER'] === NULL ||
      $_SERVER['HTTP_REFERER'] == ''
    ) {
      $return = [
        'code' => 401,
        'message' => 'No tiene permisos',
      ];
    }
    return $return;
  }

}
