<?php

namespace Drupal\tigo_portability\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\tigo_portability\Model\ZendeskTicket;
use Drupal\tigoapi\TigoApiProcess;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\tigo_portability\Services\GestorSuscripcion;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\tigo_activation\Activation;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;
use Drupal\Core\Form\FormBuilder;

/**
 * Clase para controlar el soporte de tickets.
 *
 * @package Drupal\tigo_portabilidad
 * @license https://transacciones.tigo.com.co/servicios/paquetes  License 3.01
 * @link https://transacciones.tigo.com.co/servicios/paquetes
 */
class TicketSupportController extends ControllerBase {

  protected $zendeskTicket;
  protected $tigoApi;
  protected $killSwitch;
  protected $gestor;
  protected $currentUser;
  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilder
   */
  protected $formBuilder;

  /**
   * Constructor de la clase.
   */
  public function __construct(
    ZendeskTicket $zendesk_ticket,
    TigoApiProcess $tigo_api,
    KillSwitch $kill_switch,
    GestorSuscripcion $gestor_suscripcion,
    AccountProxyInterface $user,
    FormBuilder $form_builder
  ) {
    $this->zendeskTicket = $zendesk_ticket;
    $this->tigoApi = $tigo_api;
    $this->killSwitch = $kill_switch;
    $this->gestor = $gestor_suscripcion;
    $this->currentUser = $user;
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tigo_portability.zendesk_ticket'),
      $container->get('tigoapi.process'),
      $container->get('page_cache_kill_switch'),
      $container->get('tigo_portability.package_gestor'),
      $container->get('current_user'),
      $container->get('form_builder')
    );
  }

  /**
   * Función que retorna la información al asesor del estado del ticket.
   */
  public function ticketPortedStepOne($ticket, $typeplan) {
    // Instrucción para evitar cache de valores en el Twig.
    $this->killSwitch->trigger();
    $order = $this->zendeskTicket->getPortabilityOrderByTicket($ticket);
    if ($order[0]->status == 'OPEN') {
      $statusDonor = $this->verifyTigoNumber($order[0]->linea_donante);
      if ($order[0]->tipo_plan == 'PRE') {
        $loPlan = 'Prepago';
      }
      elseif ($order[0]->tipo_plan == 'POS') {
        $loPlan = 'Pospago';
      }

      $statusTigo = $this->verifyTigoNumber($order[0]->msisdn);
      $asesor = $this->currentUser;

      $log = [
        'step' => '2: Portabilidad ' . $loPlan,
        'log' => 'Consulta estado de msisdn',
        'data' => json_encode([
          'asesor_id' => $asesor->id(),
          'Nombre asesor' => $asesor->getUsername(),
          'Msisdn donante' => $order[0]->linea_donante,
          'Status 1' => $statusDonor,
          'Msisdn Tigo' => $order[0]->msisdn,
          'Status 2' => $statusTigo,
          'Donorid' => $order[0]->donorid,
          'item_id' => $order[0]->item_id,
          'Descripción' => $order[0]->description,
          'Ticket' => $order[0]->ticket,
        ]),
      ];
      $log_encode = json_encode([$log]);
      $date = date('Y-m-d H:i:s');
      Activation::queryInsertLog(
        $date,
        $order[0]->order_id,
        $order[0]->client_email,
        $log_encode,
        'Portabilidad ' . $loPlan
      );

      $data = [
        'message' => 'La línea donante NO ha sido portada y la nueva línea aún no se encuentra activa.',
        'number' => FALSE,
        'portado' => 'NO',
        'class' => 'noline',
        'order' => $order,
        'statusDonor' => $statusDonor,
        'statusTigo' => $statusTigo,
        'ref_id' => $order[0]->suscription_id,
        'donorNumber' => $order[0]->linea_donante,
        'observaciones_active' => FALSE,
        'description' => $order[0]->description,
      ];

      if ($statusDonor == 'ACTIVE') {
        $data['message'] = 'La línea ha sido portada con exito.';
        $data['number'] = $order[0]->linea_donante;
        $data['portado'] = 'SI';
        $data['class'] = 'donor';
        $data['observaciones_active'] = TRUE;
      }
      elseif ($statusDonor == 'INACTIVE' && $statusTigo == 'ACTIVE') {
        $data['message'] = 'La línea NO ha sido portada. Se asignarán los recursos al nuevo MSISDN.';
        $data['number'] = $order[0]->msisdn;
        $data['class'] = 'tigo';
        $data['observaciones_active'] = TRUE;
      }
      if ($typeplan == 'POS') {
        $info = [
          'order_id' => $order[0]->order_id,
          'item_id' => $order[0]->item_id,
        ];
        $form = $this->formBuilder->getForm('\Drupal\tigo_activation\Form\ManualActivation\ManualActivation', $order[0]->order_id, $order[0]->item_id, $data['number']);
      }
      else {
        $info = [
          'msisdnAsig' => $data['number'],
          'order_id' => $order[0]->order_id,
          'item_id' => $order[0]->item_id,
          'typePlan' => $order[0]->tipo_plan,
          'ref_id' => $order[0]->suscription_id,
          'portado' => $data['portado'],
        ];

        $form = $this->formBuilder->getForm('\Drupal\tigo_portability\Form\SupportStatus', $info);
      }

      $build = [];
      $build['heading'] = [
        '#theme' => 'tigo-portability-ticket-ported-step1',
        '#data' => $data,
        '#attached' => [
          'library' => [
            'tigo_portability/tigo_portability_admin',
          ],
        ],
        '#cache' => ['max-age' => 0],
      ];
      $build['form'] = $form;
    }
    else {
      drupal_set_message(
        $this->t('La orden se encuentra en estado') . ' ' . $order[0]->status,
        'error'
      );
      $path = Url::fromRoute('tigo_portability.ticket_support')->toString();
      $response = new RedirectResponse($path);
      $response->send();
      exit;
    }
    return $build;
  }

  /**
   * Función que consume el servicio para verificar estado de la línea.
   *
   * @param string $msisdn
   *   Línea a ser verificada.
   *
   * @return string
   *   Retorna el estado de la línea
   */
  public function verifyTigoNumber($msisdn) {
    $type = 'account';
    $lineStatus = '';
    $numeroTigo = (object) [];
    try {
      $numeroTigo = $this->tigoApi->upselling($msisdn, $type);
      $lineStatus = (isset($numeroTigo->account->status))
      ? $numeroTigo->account->status
      : 'INACTIVE';
    }
    catch (\Exception $e) {
      $lineStatus = 'INACTIVE';
    }
    return $lineStatus;
  }

  /**
   * Función que retorna la información al asesor del estado del ticket.
   */
  public function ticketPortedStepTwo($ref_id, $msisdn, $ticket) {
    // Instrucción para evitar cache de valores en el Twig.
    $this->killSwitch->trigger();
    $order = $this->zendeskTicket->getPortabilityOrderByTicket($ticket, [
      'status',
      'observaciones',
      'package_id',
    ]);
    $typePlan = $order[0]->tipo_plan;

    $classStatus = 'succes';
    if ($typePlan == 'PRE') {
      $messageStatus = 'Se han asignado los recursos con éxito.';
    }
    else {
      $messageStatus = 'Se ha creado el contrato exitosamente.';
    }

    if ($order[0]->status == 'CANCEL') {
      $classStatus = 'warning';
      $messageStatus = 'Se ha cancelado la orden.';
    }
    elseif ($order[0]->status == 'PENDING') {
      $classStatus = 'warning';
      $messageStatus = 'Se ha presentado un problema con la orden. Se verificará el estado.';
    }

    $data = [
      'status' => $order[0]->status,
      'msisdn' => $msisdn,
      'ref_id' => $ref_id,
      'statusMessage' => $messageStatus,
      'classStatus' => $classStatus,
      'observaciones' => $order[0]->observaciones,
    ];
    return [
      '#theme' => 'tigo-portability-ticket-ported-step2',
      '#data' => $data,
      '#cache' => ['max-age' => 0],
      '#attached' => [
        'library' => [
          'tigo_portability/tigo_portability_admin',
        ],
      ],
    ];
  }

  /**
   * Función que retorna la información al asesor del estado del ticket.
   */
  public function ticketPortedCancel($ticket) {
    // Instrucción para evitar cache de valores en el Twig.
    $this->killSwitch->trigger();
    $datos = [
      'status' => 'CLOSE',
      'status_detail' => 'CANCEL',
    ];
    $this->zendeskTicket->updatePortabilityTicket($ticket, $datos);
    $data = [
      'status' => 'CANCEL',
    ];
    return [
      '#theme' => 'tigo-portability-ticket-ported-step2',
      '#data' => $data,
      '#cache' => ['max-age' => 0],
      '#attached' => [
        'library' => [
          'tigo_portability/tigo_portability_admin',
        ],
      ],
    ];
  }

}
