<?php

namespace Drupal\tigo_portability\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\tigo_portability\Model\ZendeskTicket;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\tigo_activation\Activation;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Manejo de la session php con el msisdn.
 *
 * @package Drupal\tigo_portabilidad
 * @license https://transacciones.tigo.com.co/servicios/paquetes  License 3.01
 * @link https://transacciones.tigo.com.co/servicios/paquetes
 */
class TicketSupportPortability extends ConfigFormBase {

  protected $zendeskTicket;
  protected $currentUser;

  /**
   * Constructor de la clase.
   */
  public function __construct(
    ZendeskTicket $zendesk_ticket,
    AccountProxyInterface $user
  ) {
    $this->zendeskTicket = $zendesk_ticket;
    $this->currentUser = $user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tigo_portability.zendesk_ticket'),
      $container->get('current_user')
    );
  }

  /**
   * Configuracion del formulario.
   *
   * @return array
   *   Retorna el id de configuración del formulario
   */
  public function getEditableConfigNames() {
    return [
      'tigo_portability.adminsettings',
    ];
  }

  /**
   *  Obtiene el formulario.
   *
   * @return array
   *   Retorna el id del formulario
   */
  public function getFormId() {
    return 'admin_portability_form';
  }

  /**
   * Formulario.
   *
   * @param array $form
   *   Formulario.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   Estado del Formulario.
   *
   * @return FormBase
   *   Retorna el formulario
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('tigo_portability.adminsettings');

    $form['#attached']['library'] = [
      'tigo_portability/tigo_portability_admin',
    ];

    $form['zendesk_ticket'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zendesk ticket'),
      '#description' => $this->t('Ticket de zendesk a buscar'),
      '#default_value' => $config->get('zendesk_ticket'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Buscar'),
    ];
    return $form;
  }

  /**
   * Valida los campos del formulario.
   *
   * @param array $form
   *   Formulario.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   Estado del Formulario.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $ticket = $this->zendeskTicket->getPortabilityTicket($form_state->getValue('zendesk_ticket'));
    $plan = 'Portabilidad';

    if (count($ticket) >= 1) {
      $order = $this->zendeskTicket->getPortabilityOrderByTicket($ticket[0]->ticket);
      if ($ticket[0]->status == 'OPEN') {
        $form_state->setRedirect('tigo_portability.ticket_ported_step1', [
          'ticket' => $ticket[0]->ticket,
          'typeplan' => $order[0]->tipo_producto,
        ]);
        if ($order[0]->tipo_plan == 'PRE') {
          $plan = 'Portabilidad Prepago';
        }
        elseif ($order[0]->tipo_plan == 'POS') {
          $plan = 'Portabilidad Pospago';
        }
      }
      elseif ($ticket[0]->status == 'PENDING') {
        drupal_set_message(
          $this->t('El ticket: @ticket se encuentra pendiente, se está procesando la solicitud.',
            [
              '@ticket' => $form_state->getValue('zendesk_ticket'),
            ]),
            'warning');
      }
      elseif ($ticket[0]->status == 'CANCEL') {
        drupal_set_message(
          $this->t('El ticket: @ticket fue cancelado.',
            [
              '@ticket' => $form_state->getValue('zendesk_ticket'),
            ]),
            'error');
      }
      else {
        drupal_set_message(
          $this->t('El ticket: @ticket se encuentra cerrado.',
            [
              '@ticket' => $form_state->getValue('zendesk_ticket'),
            ]));
      }

      $asesor = $this->currentUser;
      $log = [
        'step' => '1: ' . $plan,
        'log' => 'Consulta de ticket',
        'data' => json_encode([
          'asesor_id' => $asesor->id(),
          'Nombre asesor' => $asesor->getUsername(),
          'item_id' => $order[0]->item_id,
          'Ticket Zendesk' => $form_state->getValue('zendesk_ticket'),
          'Status' => $order[0]->status,
          'Documento usuario' => $order[0]->client_document,
        ]),
      ];
      $log_encode = json_encode([$log]);
      $date = date('Y-m-d H:i:s');
      Activation::queryInsertLog($date, $order[0]->order_id, $order[0]->client_email, $log_encode, $plan);
    }
    else {
      drupal_set_message(
        $this->t('El ticket: @ticket no existe.',
          [
            '@ticket' => $form_state->getValue('zendesk_ticket'),
          ]),
        'error');
    }
  }

}
