<?php

namespace Drupal\tigo_portability\Model;

use Drupal\Core\Database\Connection;

/**
 * Clase modelo para las conexiones a base de datos de tickets de zendesk.
 *
 * Se define como servicio para ser accedido en Dependency Injection.
 *
 * @package Drupal\tigo_portability
 */
class PortabilityModel {

  /**
   * Nombre de las tablas.
   *
   * @var string
   */
  const ORDERS = 'tigo_suscripcion_orders';
  const ITEMS = 'tigo_suscripcion_items';
  const PORT = 'tigo_suscripcion_portability';
  const DISPATCH = 'tigo_suscripcion_dispatch';
  const EVIDENCES = 'tigo_suscripcion_evidences';
  const SERVICE = 'tigo_suscripcion_service_request';
  const ZENDESK = 'tigo_suscripcion_zendesk';
  const PACKAGE = 'tigo_suscripcion_packages';

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;
  protected $drupal;

  /**
   * Constructor de la clase.
   */
  public function __construct(Connection $connection, $drupal = NULL) {
    $this->database = $connection;
    $this->drupal = $drupal;
    if (!is_object($this->drupal)) {
      $this->drupal = new \Drupal();
    }
  }

  /**
   * Función que busca ordenes de tipo portación en BD.
   */
  public function getPortabilityByOrderId($orderId) {
    $select = $this->database->select(self::ORDERS, 'tso');
    $select->fields('tso', ['order_id']);
    $select->addField('tsi', 'item_id');
    $select->condition('tso.order_id', $orderId);
    $select->condition('tsi.tipo_producto', 'PORT');
    $select->join(self::ITEMS, 'tsi', 'tso.order_id = tsi.order_id');
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que busca ordenes de tipo portación en BD.
   */
  public function getOrderByEmail($email) {
    $select = $this->database->select(self::ORDERS, 'tso');
    $select->fields('tso', ['order_id']);
    $select->condition('tso.client_email', $email);
    $select->condition('tsi.tipo_producto', 'PORT');
    $select->join(self::ITEMS, 'tsi', 'tso.order_id = tsi.order_id');
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que busca ordenes de tipo portación en BD.
   */
  public function getOrderDetailByEmail($email, $lineaDonante) {
    $select = $this->database->select(self::ORDERS, 'tso');
    $select->fields('tso', [
      'order_id',
      'client_name',
      'client_email',
      'client_phone',
      'address',
      'sequence',
      'creation_date',
    ]);
    $select->fields('tsi', [
      'item_id',
      'tipo_producto',
      'tipo_plan',
      'product_id',
      'suscription_id',
      'description',
    ]);
    $select->fields('tsp', [
      'donorid',
      'linea_donante',
      'nip',
      'documento',
      'tipo_documento',
      'fecha_expedicion',
    ]);
    $select->fields('tsz', [
      'ticket',
    ]);
    $select->fields('tsd', [
      'imsi',
      'msisdn',
    ]);
    $select->condition('tso.client_email', $email);
    $select->condition('tsi.tipo_producto', 'PORT');
    $select->condition('tsp.linea_donante', $lineaDonante);
    $select->join(self::ITEMS, 'tsi', 'tso.order_id = tsi.order_id');
    $select->join(self::PORT, 'tsp', 'tsi.item_id = tsp.item_id');
    $select->leftJoin(self::DISPATCH, 'tsd', 'tsi.item_id = tsd.item_id');
    $select->leftJoin(self::ZENDESK, 'tsz', 'tsi.item_id = tsz.item_id');
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que busca ordenes de tipo portación en BD.
   */
  public function getOrderByDonorId($donorId, $email) {
    $orders = $this->getOrderByEmail($email);
    if (count($orders) > 0) {
      foreach ($orders as $order) {
        $select = $this->database->select(self::PORT, 'tsp');
        $select->fields('tsp', ['linea_donante']);
        $select->condition('tsi.order_id', $order->order_id);
        $select->condition('tsp.linea_donante', $donorId);
        $select->join(self::ITEMS, 'tsi', 'tsp.item_id = tsi.item_id');
        $result = $select->execute()->fetchAll();
        if (count($result) == 0) {
          $result = (object) [
            'error' => [
              'field' => 'msisdn_other',
              'message' => 'El número de línea ingresado no es el mismo que registraste en la compra. Por favor verifícalo e intenta de nuevo.',
            ],
          ];
        }
        else {
          break;
        }
      }
    }
    else {
      $result = (object) [
        'error' => [
          'field' => 'email',
          'message' => 'El correo ingresado no es el mismo que registraste en la compra. Por favor verifícalo e intenta de nuevo.',
        ],
      ];
    }
    return $result;
  }

  /**
   * Función que busca item por item id.
   */
  public function getItemByItemId($itemId) {
    $select = $this->database->select(self::ITEMS, 'tsi');
    $select->fields('tsi');
    $select->fields('tsz', ['ticket', 'fecha_ticket']);
    $select->fields('tsp', ['linea_donante']);
    $select->fields('tsd', ['msisdn', 'imsi']);
    $select->condition('tsi.item_id', $itemId);
    $select->condition('tsi.tipo_producto', 'PORT');
    $select->join(self::ZENDESK, 'tsz', 'tsi.item_id = tsz.item_id');
    $select->join(self::PORT, 'tsp', 'tsi.item_id = tsp.item_id');
    $select->join(self::DISPATCH, 'tsd', 'tsi.item_id = tsd.item_id');
    $result = $select->execute()->fetchObject();
    return $result;
  }

  /**
   * Función que busca item con / sin ticket de Zendesk por ticket number.
   */
  public function getItemByZendeskTicket($ticket) {
    $select = $this->database->select(self::ITEMS, 'tsi');
    $select->fields('tsi');
    $select->fields('tso', [
      'order_id',
      'client_name',
      'client_document',
      'client_email',
    ]);
    $select->fields('tsp', ['linea_donante']);
    $select->fields('tsz', ['ticket', 'status', 'observaciones']);
    $select->fields('tsd', ['msisdn', 'imsi']);
    $select->condition('tsz.ticket', $ticket);
    $select->condition('tsi.tipo_producto', 'PORT');
    $select->join(self::ORDERS, 'tso', 'tsi.order_id = tso.order_id');
    $select->join(self::PORT, 'tsp', 'tsi.item_id = tsp.item_id');
    $select->join(self::ZENDESK, 'tsz', 'tsi.item_id = tsz.item_id');
    $select->join(self::DISPATCH, 'tsd', 'tsi.item_id = tsd.item_id');
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que busca item con / sin ticket de Zendesk por ticket number.
   */
  public function getItemDetailByZendeskTicket($ticket) {
    $select = $this->database->select(self::ITEMS, 'tsi');
    $select->fields('tsi', [
      'item_id',
      'product_id',
    ]);
    $select->fields('tso', [
      'client_name',
      'client_phone',
      'client_document',
      'client_email',
      'address',
      'tipo_identificacion',
    ]);
    $select->fields('tsp', [
      'fecha_expedicion',
      'tipo_documento',
      'documento',
    ]);
    $select->fields('tsd', [
      'imsi',
      'msisdn',
    ]);
    $select->fields('tspk', [
      'plan_price',
      'plan_name',
      'plan_details',
      'internet_capacidad',
      'internet_detalles',
      'show_table',
    ]);
    $select->condition('tsz.ticket', $ticket);
    $select->condition('tsi.tipo_producto', 'PORT');
    $select->join(self::ZENDESK, 'tsz', 'tsi.item_id = tsz.item_id');
    $select->join(self::ORDERS, 'tso', 'tsi.order_id = tso.order_id');
    $select->join(self::PORT, 'tsp', 'tsi.item_id = tsp.item_id');
    $select->join(self::DISPATCH, 'tsd', 'tsi.item_id = tsd.item_id');
    $select->join(self::PACKAGE, 'tspk', 'tsi.suscription_id = tspk.plan_id');
    $result = $select->execute()->fetchAll();
    return $result;
  }

}
