<?php

namespace Drupal\tigo_portability\Services;

use Drupal\tigo_vtex_manage_orders\Services\Database\ConnectVtex;
use Drupal\tigo_activation\Services\TicketTools;
use Drupal\tigo_activation\Activation;

/**
 * Validacion en el proceso de portabilidad con vtex.
 */
class ValidationVtex {
  /**
   * Variable para formatear conexion con la base de datos.
   *
   * @var \Drupal\tigo_vtex_manage_orders\Services\Database\ConnectVtex
   */
  protected $dbVtexOrders;
  /**
   * Variable para consumir servicio de zendesk.
   *
   * @var \Drupal\tigo_activation\Services\TicketTools
   */
  protected $ticketZendesk;
  /**
   * Variable para guardar datos de portabilidad.
   *
   * @var array
   */
  private $dataPortability;
  /**
   * Variable para guardar errores.
   *
   * @var array
   */
  public $error = FALSE;

  /**
   * Constructor del formulario.
   */
  public function __construct(ConnectVtex $db_vtex_orders, TicketTools $ticket_zendesk) {
    $this->dbVtexOrders = $db_vtex_orders;
    $this->ticketZendesk = $ticket_zendesk;
  }

  /**
   * Validacion formulario de portacion.
   *
   * @param string $email
   *   Correo con el que hizo la compra en vtex.
   * @param string $msisdn
   *   Linea con el que hizo la solictud de portabilidad en vtex.
   */
  public function validationPortability($email, $msisdn) {
    $msisdn = intval(preg_replace('/[^0-9]+/', '', $msisdn), 10);
    $this->dbVtexOrders->getOrdenByEmail($email, [
      'zendesk_ticket',
      'product_id',
      'imsi',
      'msisdn',
      'client_name',
      'json_address',
      'creation_date',
      'sequence',
    ], TRUE);
    $this->dbVtexOrders->joinPortability([
      'documento',
      'donorid',
      'fecha_expedicion',
      'linea_donante',
      'nip',
      'tipo_documento',
    ]);
    $result = $this->dbVtexOrders->executeQuery('object', TRUE);
    if (!$result) {
      $this->error = [
        'field' => 'email',
        'message' => 'El correo ingresado no es el mismo que registraste en la compra. Por favor verifícalo e intenta de nuevo.',
      ];
    }
    else {
      if ($result->linea_donante != $msisdn) {
        $this->error = [
          'field' => 'msisdn_other',
          'message' => 'El número de línea ingresado no es el mismo que registraste en la compra. Por favor verifícalo e intenta de nuevo.',
        ];
      }
      else {
        $this->dataPortability = (array) $result;
      }
    }
    return $this;
  }

  /**
   * Submit portabilidad.
   */
  public function submitPortability($email) {
    $redirect = FALSE;
    $pospago = FALSE;
    if (!empty($this->dataPortability['zendesk_ticket'])) {
      $new_ticket = FALSE;
      $redirect = TRUE;
    }
    else {
      if ($this->dataPortability['type_plan'] == 'PORTPOS') {
        $redirect = TRUE;
        $pospago = TRUE;
        $data = [
          'type_plan' => $this->dataPortability['type_plan'],
          'msisdn' => $this->dataPortability['linea_donante'],
          'imsi' => $this->dataPortability['imsi'],
          'docType' => $this->dataPortability['tipo_documento'],
          'docId' => $this->dataPortability['documento'],
          'year' => $this->dataPortability['fecha_expedicion'],
          'transactionId' => 'WSEC' . $this->dataPortability['sequence'] . 'P' . $this->dataPortability['item_id'],
          'item_id' => $this->dataPortability['item_id'],
          'productId' => $this->dataPortability['product_id'],
        ];
        // Valido que tipo de producto esta comprando.
        $this->dbVtexOrders->getPackages($data['productId'], ['plan_name']);
        $result = $this->dbVtexOrders->executeQuery('object');
        $data['moreData'] = [
          'name' => $this->dataPortability['client_name'],
          'real_order_id' => $this->dataPortability['order_id'],
          'email' => $email,
          'plan_id' => !empty($result->plan_id)
          ? $result->plan_id
          : FALSE,
        ];
        $_SESSION['data_pospago'] = $data;
      }
      else {
        $address = json_decode($this->dataPortability['json_address']);
        $data = [
          'order_id' => $this->dataPortability['order_id'],
          'product_id' => $this->dataPortability['product_id'],
          'donorNumber' => $this->dataPortability['linea_donante'],
          'donorId' => $this->dataPortability['donorid'],
          'asesorId' => '1019083585',
          'documentType' => $this->dataPortability['tipo_documento'],
          'documentId' => $this->dataPortability['documento'],
          'anhoExpedicion' => $this->dataPortability['fecha_expedicion'],
          'name' => $this->dataPortability['client_name'],
          'email' => $email,
          'NIP' => $this->dataPortability['nip'],
          'imsi' => $this->dataPortability['imsi'],
          'msisdn' => $this->dataPortability['msisdn'],
          'saleType' => 'chip_prepago',
          'address' => $address->street,
          'state' => $address->state,
          'city' => $address->city,
          'telefono' => !is_null($address->number)
          ? $address->number
          : $this->dataPortability['linea_donante'],
        ];
        // Llamado Zendesk.
        $returnTicket = $this->ticketZendesk->ticketPortability($data);
        if (isset($returnTicket) && is_numeric($returnTicket)) {
          $redirect = TRUE;
          $new_ticket = TRUE;
        }
      }
    }

    if ($redirect) {
      if ($pospago) {
        $return = [
          'type_plan' => $this->dataPortability['type_plan'],
        ];
        $log = [
          0 => [
            'step' => 'Portabilidad Pospago',
            'log' => 'Pospago',
            'data' => json_encode($data),
            'typePort' => 'POST'
          ],
        ];
      }
      else {
        $return = [
          'order_id' => $this->dataPortability['order_id'],
          'item_id' => $this->dataPortability['item_id'],
          'new_ticket' => $new_ticket,
        ];
        $log = [
          0 => [
            'step' => 'Portabilidad Prepago',
            'log' => $new_ticket
            ? 'Generación de ticket'
            : 'Consulta de ticket',
            'data' => json_encode([
              'Ticket' => !empty($this->dataPortability['zendesk_ticket'])
              ? $this->dataPortability['zendesk_ticket']
              : $returnTicket,
              'Operador donor' => $this->dataPortability['donorid'],
              'Línea donor' => $this->dataPortability['linea_donante'],
              'Fecha de la compra' => $this->dataPortability['creation_date'],
            ]),
            'typePort' => 'PRE'
          ],
        ];
      }
    }
    else {
      $log = [
        0 => [
          'step' => 'Portabilidad',
          'log' => 'Error con la consulta de zendesk.',
        ],
      ];
    }

    $typePort = NULL;
    if (isset($log[0]['typePort']) && $log[0]['typePort'] == 'POST') {
      $typePort = 'Portabilidad Pospago';
    }
    else {
      $typePort = 'Portabilidad Prepago';
    }
    if (isset($_SESSION['step_log']) && isset($_COOKIE['Drupal_visitor_date']) && isset($_COOKIE['Drupal_visitor_init_activation'])) {
      if (!isset($_SESSION['data_pospago'])) {
        $info = [
          'moreData' => [
            'email' => $email,
          ],
        ];
        $_SESSION['data_pospago'] = $info;
      }
      Activation::dataLogUpdate(2, $log, TRUE, $typePort);
    }
    else {
      $date = date('Y-m-d H:i:s');
      $_SESSION['step_log'] = $log;
      user_cookie_save([
        'client_email' => $email,
        'date' => $date,
      ]);
      $log_enconde = json_encode([$log]);

      Activation::queryInsertLog($date, $this->dataPortability['order_id'], $email, $log_enconde, $typePort);
    }
    user_cookie_delete('init_activation');
    return isset($return)
      ? $return
      : FALSE;
  }

}
