<?php

namespace Drupal\tigo_portability\Model;

use Drupal\Core\Database\Connection;

/**
 * Clase modelo para las conexiones a base de datos de tickets de zendesk.
 *
 * Se define como servicio para ser accedido en Dependency Injection.
 *
 * @package Drupal\tigo_portability
 */
class ZendeskTicket {

  /**
   * Nombre de la tabla.
   *
   * @var string
   */
  const ZENDESK = 'tigo_suscripcion_zendesk';
  const ORDERS = 'tigo_suscripcion_orders';
  const ITEMS = 'tigo_suscripcion_items';
  const PORT = 'tigo_suscripcion_portability';
  const DISPATCH = 'tigo_suscripcion_dispatch';
  const SERVICE = 'tigo_suscripcion_service_request';

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructor de la clase.
   */
  public function __construct(Connection $connection) {
    $this->database = $connection;
  }

  /**
   * Función que busca un ticket en BD.
   *
   * @param string $ticket
   *   Número de ticket a encontrar.
   *
   * @return array
   *   Retorna el resultado de la busqueda
   */
  public function getPortabilityTicket($ticket) {
    $select = $this->database->select(self::ZENDESK, 'tsz');
    $select->fields('tsz');
    $select->condition('tsz.ticket', $ticket);
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que busca una orden por ticket en BD.
   *
   * @param string $ticket
   *   Número de ticket a encontrar.
   *
   * @return array
   *   Retorna el resultado de la busqueda
   */
  public function getPortabilityOrderByTicket($ticket) {
    $fields_zendesk = [
      'ticket',
      'item_id',
      'tipo',
      'status',
      'observaciones',
    ];
    $fields_orders = [
      'client_name',
      'client_phone',
      'client_document',
      'client_email',
    ];
    $fields_item = [
      'order_id',
      'tipo_plan',
      'tipo_producto',
      'status',
      'ref_id',
      'description',
      'suscription_id',
    ];
    $fields_port = [
      'linea_donante',
    ];
    $fields_dispatch = [
      'msisdn',
    ];
    $select = $this->database->select(self::ZENDESK, 'tsz');
    $select->fields('tsz', $fields_zendesk);
    $select->fields('tsi', $fields_item);
    $select->fields('tso', $fields_orders);
    $select->fields('tsp', $fields_port);
    $select->fields('tsd', $fields_dispatch);
    $select->condition('tsz.ticket', $ticket);
    $select->join(self::ITEMS, 'tsi', 'tsz.item_id = tsi.item_id');
    $select->join(self::ORDERS, 'tso', 'tsi.order_id = tso.order_id');
    $select->join(self::PORT, 'tsp', 'tsi.item_id = tsp.item_id');
    $select->join(self::DISPATCH, 'tsd', 'tsi.item_id = tsd.item_id');
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que inserta un registro de ticket.
   *
   * @param array $data
   *   Array con los datos a insertar.
   *
   * @return int
   *   Retorna el id del nuevo registro
   */
  public function setPortabilityTicket(array $data) {
    $insert = $this->database->insert(self::ZENDESK);
    $insert->fields($data);
    $result = $insert->execute();
    return $result;
  }

  /**
   * Función que actualiza el estado de un ticket.
   *
   * @param string $ticket
   *   Registro a buscar para actualizar.
   * @param array $data
   *   Array con los datos a insertar.
   *
   * @return int
   *   Retorna el id de la actualización
   */
  public function updatePortabilityTicket($ticket, array $data) {
    $result_zendesk = $result_service = '';
    $datos_zendesk = [
      'status' => $data['status'],
      'observaciones' => $data['observaciones'],
      'asesor_id' => $data['asesor_id'],
    ];
    $update = $this->database->update(self::ZENDESK);
    $update->fields($datos_zendesk);
    $update->condition('ticket', $ticket);
    $result_zendesk = $update->execute();

    $message = $data['response_message'];
    $json_request = $data['json_request'];
    $json_response = $data['json_response'];

    if ($data['status'] == 'CANCEL') {
      $code = '-100';
      $message = 'Cancelado manualmente';
      $json_request = '-';
      $json_response = '-';
    }
    else {
      $code = $data['response_code'];
      if ($data['response_code'] == '210' || $data['response_code'] == '100') {
        $code = '100';
      }
      $datos_service = [
        'item_id' => $data['item_id'],
        'type' => 'PORTABILIDAD',
        'json_request' => $json_request,
        'json_response' => $json_response,
        'response_code' => $code,
        'response_message' => substr($message, 0, 255),
        'try' => 0,
        'fecha_request' => date('Y-m-d H:i:s'),
      ];

      $insert = $this->database->insert(self::SERVICE);
      $insert->fields($datos_service);
      $result_service = $insert->execute();
    }

    $result = [
      'result_zendesk' => $result_zendesk,
      'result_service' => $result_service,
    ];
    return $result;
  }

}
